@@ -1,21 +1,28 @@
 import { each, map } from 'lodash';
+
+import { DataLinkBuiltInVars, MappingType } from '@grafana/data';
+import { setDataSourceSrv } from '@grafana/runtime';
+import { config } from 'app/core/config';
+import { GRID_CELL_HEIGHT, GRID_CELL_VMARGIN } from 'app/core/constants';
+import { mockDataSource, MockDataSourceSrv } from 'app/features/alerting/unified/mocks';
+import { getPanelPlugin } from 'app/features/plugins/__mocks__/pluginMocks';
+import { MIXED_DATASOURCE_NAME } from 'app/plugins/datasource/mixed/MixedDataSource';
+
+import { VariableHide } from '../../variables/types';
 import { DashboardModel } from '../state/DashboardModel';
 import { PanelModel } from '../state/PanelModel';
-import { GRID_CELL_HEIGHT, GRID_CELL_VMARGIN } from 'app/core/constants';
-import { expect } from 'test/lib/common';
-import { DataLinkBuiltInVars, MappingType } from '@grafana/data';
-import { VariableHide } from '../../variables/types';
-import { config } from 'app/core/config';
-import { getPanelPlugin } from 'app/features/plugins/__mocks__/pluginMocks';
-import { setDataSourceSrv } from '@grafana/runtime';
-import { mockDataSource, MockDataSourceSrv } from 'app/features/alerting/unified/mocks';
-import { MIXED_DATASOURCE_NAME } from 'app/plugins/datasource/mixed/MixedDataSource';
 
 jest.mock('app/core/services/context_srv', () => ({}));
 
 const dataSources = {
   prom: mockDataSource({
     name: 'prom',
+    uid: 'prom-uid',
+    type: 'prometheus',
+  }),
+  prom2: mockDataSource({
+    name: 'prom2',
+    uid: 'prom2-uid',
     type: 'prometheus',
     isDefault: true,
   }),
@@ -186,7 +193,7 @@ describe('DashboardModel', () => {
     });
 
     it('dashboard schema version should be set to latest', () => {
-      expect(model.schemaVersion).toBe(35);
+      expect(model.schemaVersion).toBe(37);
     });
 
     it('graph thresholds should be migrated', () => {
@@ -1441,7 +1448,7 @@ describe('DashboardModel', () => {
     });
 
     it('should ignore fieldConfig.defaults', () => {
-      expect(model.panels[0].panels[0].fieldConfig.defaults).toEqual(undefined);
+      expect(model.panels[0].panels?.[0].fieldConfig.defaults).toEqual(undefined);
     });
   });
 
@@ -1741,8 +1748,8 @@ describe('DashboardModel', () => {
             },
           ],
         });
-        panel1Targets = nestedModel.panels[0].panels[0].targets;
-        panel2Targets = nestedModel.panels[0].panels[1].targets;
+        panel1Targets = nestedModel.panels[0].panels?.[0].targets;
+        panel2Targets = nestedModel.panels[0].panels?.[1].targets;
       });
 
       it('multiple stats query should have been split into one query per stat', () => {
@@ -1827,11 +1834,11 @@ describe('DashboardModel', () => {
     });
 
     it('should update panel datasource props to refs for named data source', () => {
-      expect(model.panels[0].datasource).toEqual({ type: 'prometheus', uid: 'mock-ds-2' });
+      expect(model.panels[0].datasource).toEqual({ type: 'prometheus', uid: 'prom-uid' });
     });
 
     it('should update panel datasource props to refs for default data source', () => {
-      expect(model.panels[1].datasource).toEqual(null);
+      expect(model.panels[1].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
     });
 
     it('should update panel datasource props to refs for mixed data source', () => {
@@ -1839,11 +1846,11 @@ describe('DashboardModel', () => {
     });
 
     it('should update target datasource props to refs', () => {
-      expect(model.panels[2].targets[0].datasource).toEqual({ type: 'prometheus', uid: 'mock-ds-2' });
+      expect(model.panels[2].targets[0].datasource).toEqual({ type: 'prometheus', uid: 'prom-uid' });
     });
 
     it('should update datasources in panels collapsed rows', () => {
-      expect(model.panels[3].panels[0].datasource).toEqual({ type: 'prometheus', uid: 'mock-ds-2' });
+      expect(model.panels[3].panels?.[0].datasource).toEqual({ type: 'prometheus', uid: 'prom-uid' });
     });
   });
 
@@ -1869,7 +1876,8 @@ describe('DashboardModel', () => {
       });
     });
 
-    it('should not update panel datasource to that of query level ds', () => {
+    it('should use data source on query level as source of truth', () => {
+      expect(model.panels[0].targets[0]?.datasource?.uid).toEqual('prom-not-default-uid');
       expect(model.panels[0].datasource?.uid).toEqual('prom-not-default-uid');
     });
   });
@@ -1910,6 +1918,178 @@ describe('DashboardModel', () => {
       `);
     });
   });
+
+  describe('when migrating default (null) datasource', () => {
+    let model: DashboardModel;
+
+    beforeEach(() => {
+      model = new DashboardModel({
+        templating: {
+          list: [
+            {
+              type: 'query',
+              name: 'var',
+              options: [{ text: 'A', value: 'A' }],
+              refresh: 0,
+              datasource: null,
+            },
+          ],
+        },
+        annotations: {
+          list: [
+            {
+              datasource: null,
+            },
+            {
+              datasource: 'prom',
+            },
+          ],
+        },
+        panels: [
+          {
+            id: 2,
+            datasource: null,
+            targets: [
+              {
+                datasource: null,
+              },
+            ],
+          },
+          {
+            id: 3,
+            targets: [
+              {
+                refId: 'A',
+              },
+            ],
+          },
+        ],
+        schemaVersion: 35,
+      });
+    });
+
+    it('should set data source to current default', () => {
+      expect(model.templating.list[0].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
+    });
+
+    it('should migrate annotation null query to default ds', () => {
+      expect(model.annotations.list[1].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
+    });
+
+    it('should migrate annotation query to refs', () => {
+      expect(model.annotations.list[2].datasource).toEqual({ type: 'prometheus', uid: 'prom-uid' });
+    });
+
+    it('should update panel datasource props to refs for named data source', () => {
+      expect(model.panels[0].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
+    });
+
+    it('should update panel datasource props even when undefined', () => {
+      expect(model.panels[1].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
+    });
+
+    it('should update target datasource props to refs', () => {
+      expect(model.panels[0].targets[0].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
+    });
+  });
+
+  describe('when migrating default (null) datasource with panel with expressions queries', () => {
+    let model: DashboardModel;
+
+    beforeEach(() => {
+      model = new DashboardModel({
+        panels: [
+          {
+            id: 2,
+            targets: [
+              {
+                refId: 'A',
+              },
+              {
+                refId: 'B',
+                datasource: '__expr__',
+              },
+            ],
+          },
+        ],
+        schemaVersion: 30,
+      });
+    });
+
+    it('should update panel datasource props to default datasource', () => {
+      expect(model.panels[0].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
+    });
+
+    it('should update target datasource props to default data source', () => {
+      expect(model.panels[0].targets[0].datasource).toEqual({ type: 'prometheus', uid: 'prom2-uid' });
+    });
+  });
+});
+
+describe('when generating the legend for a panel', () => {
+  let model: DashboardModel;
+
+  beforeEach(() => {
+    model = new DashboardModel({
+      panels: [
+        {
+          id: 0,
+          options: {
+            legend: {
+              displayMode: 'hidden',
+              placement: 'bottom',
+            },
+            tooltipOptions: {
+              mode: 'single',
+            },
+          },
+        },
+        {
+          id: 1,
+          options: {
+            legend: {
+              displayMode: 'list',
+              placement: 'right',
+            },
+            tooltipOptions: {
+              mode: 'single',
+            },
+          },
+        },
+        {
+          id: 2,
+          options: {
+            legend: {
+              displayMode: 'table',
+              placement: 'bottom',
+            },
+            tooltipOptions: {
+              mode: 'single',
+            },
+          },
+        },
+      ],
+      schemaVersion: 30,
+    });
+  });
+
+  it('should update displayMode = hidden to showLegend = false and displayMode = list', () => {
+    expect(model.panels[0].options.legend).toEqual({ displayMode: 'list', showLegend: false, placement: 'bottom' });
+  });
+
+  it('should keep displayMode = list and update to showLegend = true', () => {
+    expect(model.panels[1].options.legend).toEqual({ displayMode: 'list', showLegend: true, placement: 'right' });
+  });
+
+  it('should keep displayMode = table and update to showLegend = true', () => {
+    expect(model.panels[2].options.legend).toEqual({ displayMode: 'table', showLegend: true, placement: 'bottom' });
+  });
+
+  it('should preserve the placement', () => {
+    expect(model.panels[0].options.legend.placement).toEqual('bottom');
+    expect(model.panels[1].options.legend.placement).toEqual('right');
+    expect(model.panels[2].options.legend.placement).toEqual('bottom');
+  });
 });
 
 function createRow(options: any, panelDescriptions: any[]) {
