@@ -1,11 +1,5 @@
-// Libraries
 import { each, find, findIndex, flattenDeep, isArray, isBoolean, isNumber, isString, map, max, some } from 'lodash';
-// Utils
-import getFactors from 'app/core/utils/factors';
-import kbn from 'app/core/utils/kbn';
-// Types
-import { PanelModel } from './PanelModel';
-import { DashboardModel } from './DashboardModel';
+
 import {
   AnnotationQuery,
   DataLink,
@@ -29,7 +23,10 @@ import {
   ValueMap,
   ValueMapping,
 } from '@grafana/data';
-// Constants
+import { getDataSourceSrv, setDataSourceSrv } from '@grafana/runtime';
+import { AxisPlacement, GraphFieldConfig } from '@grafana/ui';
+import { getAllOptionEditors, getAllStandardFieldConfigs } from 'app/core/components/OptionsUI/registry';
+import { config } from 'app/core/config';
 import {
   DEFAULT_PANEL_SPAN,
   DEFAULT_ROW_HEIGHT,
@@ -38,23 +35,26 @@ import {
   GRID_COLUMN_COUNT,
   MIN_PANEL_HEIGHT,
 } from 'app/core/constants';
+import getFactors from 'app/core/utils/factors';
+import kbn from 'app/core/utils/kbn';
+import { DatasourceSrv } from 'app/features/plugins/datasource_srv';
 import { isConstant, isMulti } from 'app/features/variables/guard';
 import { alignCurrentWithMulti } from 'app/features/variables/shared/multiOptions';
-import { VariableHide } from '../../variables/types';
-import { config } from 'app/core/config';
-import { plugin as statPanelPlugin } from 'app/plugins/panel/stat/module';
+import { CloudWatchMetricsQuery, LegacyAnnotationQuery } from 'app/plugins/datasource/cloudwatch/types';
 import { plugin as gaugePanelPlugin } from 'app/plugins/panel/gauge/module';
-import { AxisPlacement, GraphFieldConfig } from '@grafana/ui';
-import { getDataSourceSrv } from '@grafana/runtime';
+import { plugin as statPanelPlugin } from 'app/plugins/panel/stat/module';
+
 import { labelsToFieldsTransformer } from '../../../../../packages/grafana-data/src/transformations/transformers/labelsToFields';
 import { mergeTransformer } from '../../../../../packages/grafana-data/src/transformations/transformers/merge';
 import {
   migrateCloudWatchQuery,
   migrateMultipleStatsAnnotationQuery,
   migrateMultipleStatsMetricsQuery,
-} from 'app/plugins/datasource/cloudwatch/migrations';
-import { CloudWatchAnnotationQuery, CloudWatchMetricsQuery } from 'app/plugins/datasource/cloudwatch/types';
-import { getAllOptionEditors, getAllStandardFieldConfigs } from 'app/core/components/editors/registry';
+} from '../../../plugins/datasource/cloudwatch/migrations/dashboardMigrations';
+import { VariableHide } from '../../variables/types';
+
+import { DashboardModel } from './DashboardModel';
+import { PanelModel } from './PanelModel';
 
 standardEditorsRegistry.setInit(getAllOptionEditors);
 standardFieldConfigEditorRegistry.setInit(getAllStandardFieldConfigs);
@@ -65,39 +65,10 @@ export class DashboardMigrator {
 
   constructor(dashboardModel: DashboardModel) {
     this.dashboard = dashboardModel;
-  }
 
-  /**
-   * When changing default datasource which is stored as null Grafana get's into a mixed state where queries have
-   * data source uid & type set that is different from the now new default
-   */
-  syncQueryDataSources() {
-    const dataSourceSrv = getDataSourceSrv();
-    // This only happens in some unit tests that does not set a DataSourceSrv
-    if (!dataSourceSrv) {
-      return;
-    }
-
-    const defaultDS = getDataSourceSrv().getInstanceSettings(null);
-    // if default ds is mixed then skip this
-    if (!defaultDS || defaultDS.meta.mixed) {
-      return;
-    }
-
-    for (const panel of this.dashboard.panels) {
-      // only interested in panels that use default (null) data source
-      if (panel.datasource) {
-        continue;
-      }
-
-      for (const target of panel.targets) {
-        // If query level data source is different from panel
-        if (target.datasource && target.datasource.uid !== defaultDS?.uid) {
-          // set panel level data source to data source on the query as this is more likely the correct one
-          // But impossible to say, and this changes the behavior of of what default means ahead of the big change to default
-          panel.datasource = target.datasource;
-        }
-      }
+    // for tests to pass
+    if (!getDataSourceSrv()) {
+      setDataSourceSrv(new DatasourceSrv());
     }
   }
 
@@ -105,7 +76,7 @@ export class DashboardMigrator {
     let i, j, k, n;
     const oldVersion = this.dashboard.schemaVersion;
     const panelUpgrades: PanelSchemeUpgradeHandler[] = [];
-    this.dashboard.schemaVersion = 35;
+    this.dashboard.schemaVersion = 37;
 
     if (oldVersion === this.dashboard.schemaVersion) {
       return;
@@ -736,14 +707,14 @@ export class DashboardMigrator {
     // Replace datasource name with reference, uid and type
     if (oldVersion < 33) {
       panelUpgrades.push((panel) => {
-        panel.datasource = migrateDatasourceNameToRef(panel.datasource);
+        panel.datasource = migrateDatasourceNameToRef(panel.datasource, { returnDefaultAsNull: true });
 
         if (!panel.targets) {
           return panel;
         }
 
         for (const target of panel.targets) {
-          const targetRef = migrateDatasourceNameToRef(target.datasource);
+          const targetRef = migrateDatasourceNameToRef(target.datasource, { returnDefaultAsNull: true });
           if (targetRef != null) {
             target.datasource = targetRef;
           }
@@ -766,6 +737,58 @@ export class DashboardMigrator {
       panelUpgrades.push(ensureXAxisVisibility);
     }
 
+    if (oldVersion < 36) {
+      // Migrate datasource to refs in annotations
+      for (const query of this.dashboard.annotations.list) {
+        query.datasource = migrateDatasourceNameToRef(query.datasource, { returnDefaultAsNull: false });
+      }
+
+      // Migrate datasource: null to current default
+      const defaultDs = getDataSourceSrv().getInstanceSettings(null);
+      if (defaultDs) {
+        for (const variable of this.dashboard.templating.list) {
+          if (variable.type === 'query' && variable.datasource === null) {
+            variable.datasource = getDataSourceRef(defaultDs);
+          }
+        }
+
+        panelUpgrades.push((panel: PanelModel) => {
+          if (panel.targets) {
+            let panelDataSourceWasDefault = false;
+            if (panel.datasource == null && panel.targets.length > 0) {
+              panel.datasource = getDataSourceRef(defaultDs);
+              panelDataSourceWasDefault = true;
+            }
+
+            for (const target of panel.targets) {
+              if (target.datasource == null || target.datasource.uid == null) {
+                target.datasource = { ...panel.datasource };
+              }
+
+              if (panelDataSourceWasDefault && target.datasource.uid !== '__expr__') {
+                // We can have situations when default ds changed and the panel level data source is different from the queries
+                // In this case we use the query level data source as source for truth
+                panel.datasource = target.datasource as DataSourceRef;
+              }
+            }
+          }
+          return panel;
+        });
+      }
+    }
+
+    if (oldVersion < 37) {
+      panelUpgrades.push((panel: PanelModel) => {
+        if (panel.options?.legend && panel.options.legend.displayMode === 'hidden') {
+          panel.options.legend.displayMode = 'list';
+          panel.options.legend.showLegend = false;
+        } else if (panel.options?.legend) {
+          panel.options.legend = { ...panel.options?.legend, showLegend: true };
+        }
+        return panel;
+      });
+    }
+
     if (panelUpgrades.length === 0) {
       return;
     }
@@ -773,9 +796,10 @@ export class DashboardMigrator {
     for (j = 0; j < this.dashboard.panels.length; j++) {
       for (k = 0; k < panelUpgrades.length; k++) {
         this.dashboard.panels[j] = panelUpgrades[k].call(this, this.dashboard.panels[j]);
-        if (this.dashboard.panels[j].panels) {
-          for (n = 0; n < this.dashboard.panels[j].panels.length; n++) {
-            this.dashboard.panels[j].panels[n] = panelUpgrades[k].call(this, this.dashboard.panels[j].panels[n]);
+        const rowPanels = this.dashboard.panels[j].panels;
+        if (rowPanels) {
+          for (n = 0; n < rowPanels.length; n++) {
+            rowPanels[n] = panelUpgrades[k].call(this, rowPanels[n]);
           }
         }
       }
@@ -884,7 +908,7 @@ export class DashboardMigrator {
         delete panel.span;
 
         if (rowPanelModel && rowPanel.collapsed) {
-          rowPanelModel.panels.push(panel);
+          rowPanelModel.panels?.push(panel);
         } else {
           this.dashboard.panels.push(new PanelModel(panel));
         }
@@ -1084,8 +1108,15 @@ function migrateSinglestat(panel: PanelModel) {
   return panel;
 }
 
-export function migrateDatasourceNameToRef(nameOrRef?: string | DataSourceRef | null): DataSourceRef | null {
-  if (nameOrRef == null || nameOrRef === 'default') {
+interface MigrateDatasourceNameOptions {
+  returnDefaultAsNull: boolean;
+}
+
+export function migrateDatasourceNameToRef(
+  nameOrRef: string | DataSourceRef | null | undefined,
+  options: MigrateDatasourceNameOptions
+): DataSourceRef | null {
+  if (options.returnDefaultAsNull && (nameOrRef == null || nameOrRef === 'default')) {
     return null;
   }
 
@@ -1152,7 +1183,9 @@ function isCloudWatchQuery(target: DataQuery): target is CloudWatchMetricsQuery
   );
 }
 
-function isLegacyCloudWatchAnnotationQuery(target: AnnotationQuery<DataQuery>): target is CloudWatchAnnotationQuery {
+function isLegacyCloudWatchAnnotationQuery(
+  target: AnnotationQuery<DataQuery>
+): target is AnnotationQuery<LegacyAnnotationQuery> {
   return (
     target.hasOwnProperty('dimensions') &&
     target.hasOwnProperty('namespace') &&
