@@ -1,10 +1,13 @@
+import { render } from '@testing-library/react';
 import React from 'react';
-import { shallow } from 'enzyme';
+import { mockToolkitActionCreator } from 'test/core/redux/mocks';
+
 import { NavModel } from '@grafana/data';
 
-import { OrgDetailsPage, Props } from './OrgDetailsPage';
+import { backendSrv } from '../../core/services/backend_srv';
 import { Organization } from '../../types';
-import { mockToolkitActionCreator } from 'test/core/redux/mocks';
+
+import { OrgDetailsPage, Props } from './OrgDetailsPage';
 import { setOrganizationName } from './state/reducers';
 
 jest.mock('app/core/core', () => {
@@ -15,7 +18,28 @@ jest.mock('app/core/core', () => {
   };
 });
 
+jest.mock('@grafana/runtime', () => {
+  const originalModule = jest.requireActual('@grafana/runtime');
+  return {
+    ...originalModule,
+    config: {
+      ...originalModule.config,
+      featureToggles: {
+        internationalization: true,
+      },
+    },
+  };
+});
+
 const setup = (propOverrides?: object) => {
+  jest.clearAllMocks();
+  // needed because SharedPreferences is rendered in the test
+  jest.spyOn(backendSrv, 'put');
+  jest
+    .spyOn(backendSrv, 'get')
+    .mockResolvedValue({ timezone: 'UTC', homeDashboardUID: 'home-dashboard', theme: 'dark' });
+  jest.spyOn(backendSrv, 'search').mockResolvedValue([]);
+
   const props: Props = {
     organization: {} as Organization,
     navModel: {
@@ -30,32 +54,30 @@ const setup = (propOverrides?: object) => {
     setOrganizationName: mockToolkitActionCreator(setOrganizationName),
     updateOrganization: jest.fn(),
   };
-
   Object.assign(props, propOverrides);
 
-  return shallow(<OrgDetailsPage {...props} />);
+  render(<OrgDetailsPage {...props} />);
 };
 
 describe('Render', () => {
   it('should render component', () => {
-    const wrapper = setup();
-
-    expect(wrapper).toMatchSnapshot();
+    expect(() => setup()).not.toThrow();
   });
 
   it('should render organization and preferences', () => {
-    const wrapper = setup({
-      organization: {
-        name: 'Cool org',
-        id: 1,
-      },
-      preferences: {
-        homeDashboardId: 1,
-        theme: 'Default',
-        timezone: 'Default',
-      },
-    });
-
-    expect(wrapper).toMatchSnapshot();
+    expect(() =>
+      setup({
+        organization: {
+          name: 'Cool org',
+          id: 1,
+        },
+        preferences: {
+          homeDashboardUID: 'home-dashboard',
+          theme: 'Default',
+          timezone: 'Default',
+          locale: '',
+        },
+      })
+    ).not.toThrow();
   });
 });
