@@ -1,9 +1,11 @@
+import { render, screen } from '@testing-library/react';
+import userEvent from '@testing-library/user-event';
 import React from 'react';
-import { shallow } from 'enzyme';
-import UsersTable, { Props } from './UsersTable';
+
 import { OrgUser } from 'app/types';
+
+import UsersTable, { Props } from './UsersTable';
 import { getMockUsers } from './__mocks__/userMocks';
-import { ConfirmModal } from '@grafana/ui';
 
 jest.mock('app/core/core', () => ({
   contextSrv: {
@@ -22,30 +24,32 @@ const setup = (propOverrides?: object) => {
 
   Object.assign(props, propOverrides);
 
-  return shallow(<UsersTable {...props} />);
+  render(<UsersTable {...props} />);
 };
 
 describe('Render', () => {
   it('should render component', () => {
-    const wrapper = setup();
-
-    expect(wrapper).toMatchSnapshot();
+    expect(() => setup()).not.toThrow();
   });
 
-  it('should render users table', () => {
-    const wrapper = setup({
-      users: getMockUsers(5),
-    });
+  it('should render users in table', () => {
+    const usersData = getMockUsers(5);
+    setup({ users: usersData });
 
-    expect(wrapper).toMatchSnapshot();
+    usersData.forEach((user) => {
+      expect(screen.getByText(user.name)).toBeInTheDocument();
+    });
   });
 });
 
 describe('Remove modal', () => {
-  it('should render correct amount', () => {
-    const wrapper = setup({
-      users: getMockUsers(3),
-    });
-    expect(wrapper.find(ConfirmModal).length).toEqual(0);
+  it('should render confirm check on delete', async () => {
+    const usersData = getMockUsers(3);
+    setup({ users: usersData });
+    const user = userEvent.setup();
+
+    await user.click(screen.getAllByRole('button', { name: /delete/i })[0]);
+
+    expect(screen.getByText(/are you sure/i)).toBeInTheDocument();
   });
 });
