@@ -1,8 +1,12 @@
+import React from 'react';
+
 import { DataSourcePluginOptionsEditorProps } from '@grafana/data';
+import { config } from '@grafana/runtime';
 import { DataSourceHttpSettings } from '@grafana/ui';
+import { SpanBarSettings } from '@jaegertracing/jaeger-ui-components';
 import { NodeGraphSettings } from 'app/core/components/NodeGraphSettings';
 import { TraceToLogsSettings } from 'app/core/components/TraceToLogs/TraceToLogsSettings';
-import React from 'react';
+import { TraceToMetricsSettings } from 'app/core/components/TraceToMetrics/TraceToMetricsSettings';
 
 export type Props = DataSourcePluginOptionsEditorProps;
 
@@ -19,9 +23,20 @@ export const ConfigEditor: React.FC<Props> = ({ options, onOptionsChange }) => {
       <div className="gf-form-group">
         <TraceToLogsSettings options={options} onOptionsChange={onOptionsChange} />
       </div>
+
+      {config.featureToggles.traceToMetrics ? (
+        <div className="gf-form-group">
+          <TraceToMetricsSettings options={options} onOptionsChange={onOptionsChange} />
+        </div>
+      ) : null}
+
       <div className="gf-form-group">
         <NodeGraphSettings options={options} onOptionsChange={onOptionsChange} />
       </div>
+
+      <div className="gf-form-group">
+        <SpanBarSettings options={options} onOptionsChange={onOptionsChange} />
+      </div>
     </>
   );
 };
