@@ -1,7 +1,7 @@
 import { once, chain, difference } from 'lodash';
 import LRU from 'lru-cache';
-import { Value } from 'slate';
 import Prism from 'prismjs';
+import { Value } from 'slate';
 
 import {
   AbstractLabelMatcher,
@@ -11,12 +11,13 @@ import {
   HistoryItem,
   LanguageProvider,
 } from '@grafana/data';
+import { BackendSrvRequest } from '@grafana/runtime';
 import { CompletionItem, CompletionItemGroup, SearchFunctionType, TypeaheadInput, TypeaheadOutput } from '@grafana/ui';
 
+import { PrometheusDatasource } from './datasource';
 import {
   addLimitInfo,
   extractLabelMatchers,
-  fixSummariesMetadata,
   parseSelector,
   processHistogramMetrics,
   processLabels,
@@ -24,8 +25,6 @@ import {
   toPromLikeQuery,
 } from './language_utils';
 import PromqlSyntax, { FUNCTIONS, RATE_RANGES } from './promql';
-
-import { PrometheusDatasource } from './datasource';
 import { PromMetricsMetadata, PromQuery } from './types';
 
 const DEFAULT_KEYS = ['job', 'instance'];
@@ -63,12 +62,19 @@ export function addHistoryMetadata(item: CompletionItem, history: any[]): Comple
 function addMetricsMetadata(metric: string, metadata?: PromMetricsMetadata): CompletionItem {
   const item: CompletionItem = { label: metric };
   if (metadata && metadata[metric]) {
-    const { type, help } = metadata[metric];
-    item.documentation = `${type.toUpperCase()}: ${help}`;
+    item.documentation = getMetadataString(metric, metadata);
   }
   return item;
 }
 
+export function getMetadataString(metric: string, metadata: PromMetricsMetadata): string | undefined {
+  if (!metadata[metric]) {
+    return undefined;
+  }
+  const { type, help } = metadata[metric];
+  return `${type.toUpperCase()}: ${help}`;
+}
+
 const PREFIX_DELIMITER_REGEX =
   /(="|!="|=~"|!~"|\{|\[|\(|\+|-|\/|\*|%|\^|\band\b|\bor\b|\bunless\b|==|>=|!=|<=|>|<|=|~|,)/;
 
@@ -90,7 +96,7 @@ export default class PromQlLanguageProvider extends LanguageProvider {
    *  not account for different size of a response. If that is needed a `length` function can be added in the options.
    *  10 as a max size is totally arbitrary right now.
    */
-  private labelsCache = new LRU<string, Record<string, string[]>>(10);
+  private labelsCache = new LRU<string, Record<string, string[]>>({ max: 10 });
 
   constructor(datasource: PrometheusDatasource, initialValues?: Partial<PromQlLanguageProvider>) {
     super();
@@ -114,9 +120,9 @@ export default class PromQlLanguageProvider extends LanguageProvider {
     return PromqlSyntax;
   }
 
-  request = async (url: string, defaultValue: any, params = {}): Promise<any> => {
+  request = async (url: string, defaultValue: any, params = {}, options?: Partial<BackendSrvRequest>): Promise<any> => {
     try {
-      const res = await this.datasource.metadataRequest(url, params);
+      const res = await this.datasource.metadataRequest(url, params, options);
       return res.data.data;
     } catch (error) {
       console.error(error);
@@ -133,11 +139,27 @@ export default class PromQlLanguageProvider extends LanguageProvider {
     // TODO #33976: make those requests parallel
     await this.fetchLabels();
     this.metrics = (await this.fetchLabelValues('__name__')) || [];
-    this.metricsMetadata = fixSummariesMetadata(await this.request('/api/v1/metadata', {}));
+    await this.loadMetricsMetadata();
     this.histogramMetrics = processHistogramMetrics(this.metrics).sort();
     return [];
   };
 
+  async loadMetricsMetadata() {
+    // The metadata endpoint is experimental and
+    // we have customers who are not implementing it.
+    // This is to be a temporary fix until the Observability Metrics Squad
+    // has time to implement the 2022Q3 plan
+    // to detect prometheus versions and implementations.
+    // This will allow us to see if endpoints such as api/v1/metadata
+    // are implemented or not and handle them as such
+    this.metricsMetadata = {};
+
+    // PREVIOUS IMPLEMENTATION FOR NOTES
+    // this.metricsMetadata = fixSummariesMetadata(
+    //   await this.request('/api/v1/metadata', {}, {}, { showErrorAlert: false })
+    // );
+  }
+
   getLabelKeys(): string[] {
     return this.labelKeys;
   }
@@ -460,7 +482,7 @@ export default class PromQlLanguageProvider extends LanguageProvider {
 
   fetchLabelValues = async (key: string): Promise<string[]> => {
     const params = this.datasource.getTimeRangeParams();
-    const url = `/api/v1/label/${key}/values`;
+    const url = `/api/v1/label/${this.datasource.interpolateString(key)}/values`;
     return await this.request(url, [], params);
   };
 
@@ -491,10 +513,11 @@ export default class PromQlLanguageProvider extends LanguageProvider {
    * @param withName
    */
   fetchSeriesLabels = async (name: string, withName?: boolean): Promise<Record<string, string[]>> => {
+    const interpolatedName = this.datasource.interpolateString(name);
     const range = this.datasource.getTimeRangeParams();
     const urlParams = {
       ...range,
-      'match[]': name,
+      'match[]': interpolatedName,
     };
     const url = `/api/v1/series`;
     // Cache key is a bit different here. We add the `withName` param and also round up to a minute the intervals.
@@ -502,7 +525,7 @@ export default class PromQlLanguageProvider extends LanguageProvider {
     // millisecond while still actually getting all the keys for the correct interval. This still can create problems
     // when user does not the newest values for a minute if already cached.
     const cacheParams = new URLSearchParams({
-      'match[]': name,
+      'match[]': interpolatedName,
       start: roundSecToMin(parseInt(range.start, 10)).toString(),
       end: roundSecToMin(parseInt(range.end, 10)).toString(),
       withName: withName ? 'true' : 'false',
