@@ -1,11 +1,16 @@
-import { DataSourcePluginOptionsEditorProps } from '@grafana/data';
-import { DataSourceHttpSettings } from '@grafana/ui';
-import { TraceToLogsSettings } from 'app/core/components/TraceToLogs/TraceToLogsSettings';
 import React from 'react';
-import { ServiceGraphSettings } from './ServiceGraphSettings';
+
+import { DataSourcePluginOptionsEditorProps } from '@grafana/data';
 import { config } from '@grafana/runtime';
-import { SearchSettings } from './SearchSettings';
+import { DataSourceHttpSettings } from '@grafana/ui';
+import { SpanBarSettings } from '@jaegertracing/jaeger-ui-components';
 import { NodeGraphSettings } from 'app/core/components/NodeGraphSettings';
+import { TraceToLogsSettings } from 'app/core/components/TraceToLogs/TraceToLogsSettings';
+import { TraceToMetricsSettings } from 'app/core/components/TraceToMetrics/TraceToMetricsSettings';
+
+import { LokiSearchSettings } from './LokiSearchSettings';
+import { SearchSettings } from './SearchSettings';
+import { ServiceGraphSettings } from './ServiceGraphSettings';
 
 export type Props = DataSourcePluginOptionsEditorProps;
 
@@ -22,19 +27,32 @@ export const ConfigEditor: React.FC<Props> = ({ options, onOptionsChange }) => {
       <div className="gf-form-group">
         <TraceToLogsSettings options={options} onOptionsChange={onOptionsChange} />
       </div>
-      {config.featureToggles.tempoServiceGraph && (
+
+      {config.featureToggles.traceToMetrics ? (
         <div className="gf-form-group">
-          <ServiceGraphSettings options={options} onOptionsChange={onOptionsChange} />
+          <TraceToMetricsSettings options={options} onOptionsChange={onOptionsChange} />
         </div>
-      )}
-      {config.featureToggles.tempoSearch && (
-        <div className="gf-form-group">
-          <SearchSettings options={options} onOptionsChange={onOptionsChange} />
-        </div>
-      )}
+      ) : null}
+
+      <div className="gf-form-group">
+        <ServiceGraphSettings options={options} onOptionsChange={onOptionsChange} />
+      </div>
+
+      <div className="gf-form-group">
+        <SearchSettings options={options} onOptionsChange={onOptionsChange} />
+      </div>
+
       <div className="gf-form-group">
         <NodeGraphSettings options={options} onOptionsChange={onOptionsChange} />
       </div>
+
+      <div className="gf-form-group">
+        <LokiSearchSettings options={options} onOptionsChange={onOptionsChange} />
+      </div>
+
+      <div className="gf-form-group">
+        <SpanBarSettings options={options} onOptionsChange={onOptionsChange} />
+      </div>
     </>
   );
 };
