@@ -1,10 +1,10 @@
 import { PanelPlugin } from '@grafana/data';
+import { FrameState } from 'app/features/canvas/runtime/frame';
 
 import { CanvasPanel, InstanceState } from './CanvasPanel';
-import { PanelOptions } from './models.gen';
 import { getElementEditor } from './editor/elementEditor';
 import { getLayerEditor } from './editor/layerEditor';
-import { GroupState } from 'app/features/canvas/runtime/group';
+import { PanelOptions } from './models.gen';
 
 export const plugin = new PanelPlugin<PanelOptions>(CanvasPanel)
   .setNoPadding() // extend to panel edges
@@ -19,13 +19,20 @@ export const plugin = new PanelPlugin<PanelOptions>(CanvasPanel)
       defaultValue: true,
     });
 
+    builder.addBooleanSwitch({
+      path: 'showAdvancedTypes',
+      name: 'Show advanced element types',
+      description: '',
+      defaultValue: false,
+    });
+
     if (state) {
       builder.addNestedOptions(getLayerEditor(state));
 
       const selection = state.selected;
       if (selection?.length === 1) {
         const element = selection[0];
-        if (!(element instanceof GroupState)) {
+        if (!(element instanceof FrameState)) {
           builder.addNestedOptions(
             getElementEditor({
               category: [`Selected element (${element.options.name})`],
